@@ -273,6 +273,10 @@ class NodeNotFound(NotFound):
     _msg_fmt = _("Node %(node)s could not be found.")
 
 
+class DuplicateNodeOnLookup(NodeNotFound):
+    pass  # Same error message, the difference only matters internally
+
+
 class PortgroupNotFound(NotFound):
     _msg_fmt = _("Portgroup %(portgroup)s could not be found.")
 
@@ -563,6 +567,10 @@ class NodeCleaningFailure(IronicException):
     _msg_fmt = _("Failed to clean node %(node)s: %(reason)s")
 
 
+class NodeServicingFailure(IronicException):
+    _msg_fmt = _("Failed to service node %(node)s: %(reason)s")
+
+
 class PathNotFound(IronicException):
     _msg_fmt = _("Path %(dir)s does not exist.")
 
@@ -647,10 +655,6 @@ class InstanceUnrescueFailure(IronicException):
                  '%(node)s: %(reason)s')
 
 
-class XClarityError(IronicException):
-    _msg_fmt = _("XClarity exception occurred. Error: %(error)s")
-
-
 class BIOSSettingAlreadyExists(Conflict):
     _msg_fmt = _('A BIOS setting %(name)s for node %(node)s already exists.')
 
@@ -720,14 +724,6 @@ class InvalidKickstartFile(Invalid):
     _msg_fmt = _("The kickstart file is not valid.")
 
 
-class IBMCError(DriverOperationError):
-    _msg_fmt = _("IBMC exception occurred on node %(node)s. Error: %(error)s")
-
-
-class IBMCConnectionError(IBMCError):
-    _msg_fmt = _("IBMC connection failed for node %(node)s: %(error)s")
-
-
 class ClientSideError(RuntimeError):
     def __init__(self, msg=None, status_code=400, faultcode='Client'):
         self.msg = msg
@@ -891,6 +887,10 @@ class UnsupportedHardwareFeature(Invalid):
                  "requested configuration.")
 
 
+class BootModeNotAllowed(Invalid):
+    _msg_fmt = _("'%(mode)s' boot mode is not allowed for %(op)s operation.")
+
+
 class InvalidImage(ImageUnacceptable):
     _msg_fmt = _("The requested image is not valid for use.")
 
