@@ -111,9 +111,9 @@ class DoNodeDeployTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
             self.assertEqual(states.ACTIVE, node.target_provision_state)
             self.assertIsNotNone(node.last_error)
             if unexpected:
-                self.assertIn('Exception', node.last_error)
+                self.assertIn(exc.__class__.__name__, node.last_error)
             else:
-                self.assertNotIn('Exception', node.last_error)
+                self.assertNotIn(exc.__class__.__name__, node.last_error)
 
             mock_deploy.assert_called_once_with(mock.ANY, task)
 
@@ -174,63 +174,6 @@ class DoNodeDeployTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
     def test__do_node_deploy_fast_track(self):
         self._test__do_node_deploy_ok(fast_track=True)
 
-    @mock.patch('openstack.baremetal.configdrive.build', autospec=True)
-    def test__do_node_deploy_configdrive_as_dict(self, mock_cd):
-        mock_cd.return_value = 'foo'
-        configdrive = {'user_data': 'abcd'}
-        self._test__do_node_deploy_ok(configdrive=configdrive,
-                                      expected_configdrive='foo')
-        mock_cd.assert_called_once_with({'uuid': self.node.uuid},
-                                        network_data=None,
-                                        user_data=b'abcd',
-                                        vendor_data=None)
-
-    @mock.patch('openstack.baremetal.configdrive.build', autospec=True)
-    def test__do_node_deploy_configdrive_as_dict_with_meta_data(self, mock_cd):
-        mock_cd.return_value = 'foo'
-        configdrive = {'meta_data': {'uuid': uuidutils.generate_uuid(),
-                                     'name': 'new-name',
-                                     'hostname': 'example.com'}}
-        self._test__do_node_deploy_ok(configdrive=configdrive,
-                                      expected_configdrive='foo')
-        mock_cd.assert_called_once_with(configdrive['meta_data'],
-                                        network_data=None,
-                                        user_data=None,
-                                        vendor_data=None)
-
-    @mock.patch('openstack.baremetal.configdrive.build', autospec=True)
-    def test__do_node_deploy_configdrive_with_network_data(self, mock_cd):
-        mock_cd.return_value = 'foo'
-        configdrive = {'network_data': {'links': []}}
-        self._test__do_node_deploy_ok(configdrive=configdrive,
-                                      expected_configdrive='foo')
-        mock_cd.assert_called_once_with({'uuid': self.node.uuid},
-                                        network_data={'links': []},
-                                        user_data=None,
-                                        vendor_data=None)
-
-    @mock.patch('openstack.baremetal.configdrive.build', autospec=True)
-    def test__do_node_deploy_configdrive_and_user_data_as_dict(self, mock_cd):
-        mock_cd.return_value = 'foo'
-        configdrive = {'user_data': {'user': 'data'}}
-        self._test__do_node_deploy_ok(configdrive=configdrive,
-                                      expected_configdrive='foo')
-        mock_cd.assert_called_once_with({'uuid': self.node.uuid},
-                                        network_data=None,
-                                        user_data=b'{"user": "data"}',
-                                        vendor_data=None)
-
-    @mock.patch('openstack.baremetal.configdrive.build', autospec=True)
-    def test__do_node_deploy_configdrive_with_vendor_data(self, mock_cd):
-        mock_cd.return_value = 'foo'
-        configdrive = {'vendor_data': {'foo': 'bar'}}
-        self._test__do_node_deploy_ok(configdrive=configdrive,
-                                      expected_configdrive='foo')
-        mock_cd.assert_called_once_with({'uuid': self.node.uuid},
-                                        network_data=None,
-                                        user_data=None,
-                                        vendor_data={'foo': 'bar'})
-
     @mock.patch.object(swift, 'SwiftAPI', autospec=True)
     @mock.patch('ironic.drivers.modules.fake.FakeDeploy.prepare',
                 autospec=True)
@@ -248,7 +191,7 @@ class DoNodeDeployTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
         self.assertRaises(exception.SwiftOperationError,
                           deployments.do_node_deploy, task,
                           self.service.conductor.id,
-                          configdrive=b'fake config drive')
+                          configdrive='fake config drive')
         node.refresh()
         self.assertEqual(states.DEPLOYFAIL, node.provision_state)
         self.assertEqual(states.ACTIVE, node.target_provision_state)
@@ -273,8 +216,8 @@ class DoNodeDeployTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
             self.assertRaises(db_exception.DBDataError,
                               deployments.do_node_deploy, task,
                               self.service.conductor.id,
-                              configdrive=b'fake config drive')
-            expected_instance_info.update(configdrive=b'fake config drive')
+                              configdrive='fake config drive')
+            expected_instance_info.update(configdrive='fake config drive')
             expected_calls = [
                 mock.call(node.uuid,
                           {'version': mock.ANY,
@@ -310,7 +253,7 @@ class DoNodeDeployTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
         self.assertRaises(RuntimeError,
                           deployments.do_node_deploy, task,
                           self.service.conductor.id,
-                          configdrive=b'fake config drive')
+                          configdrive='fake config drive')
         node.refresh()
         self.assertEqual(states.DEPLOYFAIL, node.provision_state)
         self.assertEqual(states.ACTIVE, node.target_provision_state)
@@ -339,6 +282,56 @@ class DoNodeDeployTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
             self.assertIsNone(node.last_error)
             mock_deploy.assert_called_once_with(mock.ANY, mock.ANY)
 
+    def test_node_validation_in_disabled_bios_boot_mode_fails(self):
+        self.config(disallowed_deployment_boot_modes=['bios'],
+                    group='conductor')
+        node = obj_utils.create_test_node(self.context,
+                                          properties={'boot_mode': 'bios'},
+                                          driver='fake-hardware')
+
+        with task_manager.acquire(self.context, node.uuid,
+                                  shared=False) as task:
+            self.assertRaises(exception.BootModeNotAllowed,
+                              deployments.validate_node,
+                              task, event='deploy')
+
+    def test_node_validation_in_disabled_uefi_boot_mode_fails(self):
+        self.config(disallowed_deployment_boot_modes=['uefi'],
+                    group='conductor')
+        node = obj_utils.create_test_node(self.context,
+                                          properties={'boot_mode': 'uefi'},
+                                          driver='fake-hardware')
+
+        with task_manager.acquire(self.context, node.uuid,
+                                  shared=False) as task:
+            self.assertRaises(exception.BootModeNotAllowed,
+                              deployments.validate_node,
+                              task, event='deploy')
+
+    def test_update_fails_on_invalid_boot_mode(self):
+        # NOTE(cid): This test might need updating if boot modes' naming
+        # convention changes
+        self.assertRaises(ValueError,
+                          self.config,
+                          disallowed_deployment_boot_modes=['BIOS'],
+                          group='conductor')
+        self.assertRaises(ValueError,
+                          self.config,
+                          disallowed_deployment_boot_modes=['Bios'],
+                          group='conductor')
+        self.assertRaises(ValueError,
+                          self.config,
+                          disallowed_deployment_boot_modes=['UEFI'],
+                          group='conductor')
+        self.assertRaises(ValueError,
+                          self.config,
+                          disallowed_deployment_boot_modes=['Uefi'],
+                          group='conductor')
+        self.assertRaises(ValueError,
+                          self.config,
+                          disallowed_deployment_boot_modes=['blah'],
+                          group='conductor')
+
     @mock.patch.object(deployments, 'do_next_deploy_step', autospec=True)
     @mock.patch.object(conductor_steps, 'set_node_deployment_steps',
                        autospec=True)
@@ -394,10 +387,16 @@ class DoNodeDeployTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
     @mock.patch.object(conductor_utils, 'validate_instance_info_traits',
                        autospec=True)
     @mock.patch.object(images, 'is_whole_disk_image', autospec=True)
-    def test_start_deploy(self, mock_iwdi, mock_validate_traits,
-                          mock_validate_deploy_user_steps_and_templates,
-                          mock_deploy_validate,
-                          mock_power_validate, mock_process_event):
+    def _test_start_deploy(self, mock_iwdi, mock_validate_traits,
+                           mock_validate_deploy_user_steps_and_templates,
+                           mock_deploy_validate,
+                           mock_power_validate, mock_process_event,
+                           automatic_lessee=False):
+        self.context.auth_token_info = {
+            'token': {'project': {'id': 'user-project'}}
+        }
+        if automatic_lessee:
+            self.config(automatic_lessee=True, group='conductor')
         self._start_service()
         mock_iwdi.return_value = False
         deploy_steps = [{"interface": "bios", "step": "factory_reset",
@@ -410,7 +409,120 @@ class DoNodeDeployTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
         deployments.start_deploy(task, self.service, configdrive=None,
                                  event='deploy', deploy_steps=deploy_steps)
         node.refresh()
-        self.assertTrue(mock_iwdi.called)
+        mock_iwdi.assert_called_once_with(task.context,
+                                          task.node.instance_info)
+        self.assertFalse(node.driver_internal_info['is_whole_disk_image'])
+        self.assertEqual('partition', node.instance_info['image_type'])
+        mock_power_validate.assert_called_once_with(task.driver.power, task)
+        mock_deploy_validate.assert_called_once_with(task.driver.deploy, task)
+        mock_validate_traits.assert_called_once_with(task.node)
+        mock_validate_deploy_user_steps_and_templates.assert_called_once_with(
+            task, deploy_steps, skip_missing=True)
+        mock_process_event.assert_called_with(
+            mock.ANY, 'deploy', call_args=(
+                deployments.do_node_deploy, task, 1, None, deploy_steps),
+            callback=mock.ANY, err_handler=mock.ANY)
+        if automatic_lessee:
+            self.assertEqual('user-project', node.lessee)
+            self.assertIn('automatic_lessee', node.driver_internal_info)
+        else:
+            self.assertIsNone(node.lessee)
+            self.assertNotIn('automatic_lessee', node.driver_internal_info)
+
+    def test_start_deploy(self):
+        self._test_start_deploy(automatic_lessee=False)
+
+    def test_start_deploy_records_lessee(self):
+        self._test_start_deploy(automatic_lessee=True)
+
+    @mock.patch.object(images, 'is_source_a_path', autospec=True)
+    @mock.patch.object(task_manager.TaskManager, 'process_event',
+                       autospec=True)
+    @mock.patch('ironic.drivers.modules.fake.FakePower.validate',
+                autospec=True)
+    @mock.patch('ironic.drivers.modules.fake.FakeDeploy.validate',
+                autospec=True)
+    @mock.patch.object(conductor_steps,
+                       'validate_user_deploy_steps_and_templates',
+                       autospec=True)
+    @mock.patch.object(conductor_utils, 'validate_instance_info_traits',
+                       autospec=True)
+    @mock.patch.object(images, 'is_whole_disk_image', autospec=True)
+    def test_start_deploy_source_path(
+            self, mock_iwdi, mock_validate_traits,
+            mock_validate_deploy_user_steps_and_templates,
+            mock_deploy_validate, mock_power_validate,
+            mock_process_event, mock_is_source_a_path):
+        self._start_service()
+        mock_iwdi.return_value = None
+        mock_is_source_a_path.return_value = True
+        deploy_steps = [{"interface": "bios", "step": "factory_reset",
+                         "priority": 95}]
+        node = obj_utils.create_test_node(self.context, driver='fake-hardware',
+                                          provision_state=states.AVAILABLE,
+                                          target_provision_state=states.ACTIVE)
+        task = task_manager.TaskManager(self.context, node.uuid)
+
+        deployments.start_deploy(task, self.service, configdrive=None,
+                                 event='deploy', deploy_steps=deploy_steps)
+        node.refresh()
+        mock_iwdi.assert_called_once_with(task.context,
+                                          task.node.instance_info)
+        mock_is_source_a_path.assert_called_once_with(
+            task.context,
+            task.node.instance_info.get('image_source')
+        )
+        self.assertNotIn('is_whole_disk_iamge', node.driver_internal_info)
+        self.assertTrue(node.driver_internal_info['is_source_a_path'])
+        mock_power_validate.assert_called_once_with(task.driver.power, task)
+        mock_deploy_validate.assert_called_once_with(task.driver.deploy, task)
+        mock_validate_traits.assert_called_once_with(task.node)
+        mock_validate_deploy_user_steps_and_templates.assert_called_once_with(
+            task, deploy_steps, skip_missing=True)
+        mock_process_event.assert_called_with(
+            mock.ANY, 'deploy', call_args=(
+                deployments.do_node_deploy, task, 1, None, deploy_steps),
+            callback=mock.ANY, err_handler=mock.ANY)
+
+    @mock.patch.object(images, 'is_source_a_path', autospec=True)
+    @mock.patch.object(task_manager.TaskManager, 'process_event',
+                       autospec=True)
+    @mock.patch('ironic.drivers.modules.fake.FakePower.validate',
+                autospec=True)
+    @mock.patch('ironic.drivers.modules.fake.FakeDeploy.validate',
+                autospec=True)
+    @mock.patch.object(conductor_steps,
+                       'validate_user_deploy_steps_and_templates',
+                       autospec=True)
+    @mock.patch.object(conductor_utils, 'validate_instance_info_traits',
+                       autospec=True)
+    @mock.patch.object(images, 'is_whole_disk_image', autospec=True)
+    def test_start_deploy_source_path_none(
+            self, mock_iwdi, mock_validate_traits,
+            mock_validate_deploy_user_steps_and_templates,
+            mock_deploy_validate, mock_power_validate,
+            mock_process_event, mock_is_source_a_path):
+        self._start_service()
+        mock_iwdi.return_value = None
+        mock_is_source_a_path.return_value = None
+        deploy_steps = [{"interface": "bios", "step": "factory_reset",
+                         "priority": 95}]
+        node = obj_utils.create_test_node(self.context, driver='fake-hardware',
+                                          provision_state=states.AVAILABLE,
+                                          target_provision_state=states.ACTIVE)
+        task = task_manager.TaskManager(self.context, node.uuid)
+
+        deployments.start_deploy(task, self.service, configdrive=None,
+                                 event='deploy', deploy_steps=deploy_steps)
+        node.refresh()
+        mock_iwdi.assert_called_once_with(task.context,
+                                          task.node.instance_info)
+        mock_is_source_a_path.assert_called_once_with(
+            task.context,
+            task.node.instance_info.get('image_source')
+        )
+        self.assertNotIn('is_whole_disk_iamge', node.driver_internal_info)
+        self.assertNotIn('is_source_a_path', node.driver_internal_info)
         mock_power_validate.assert_called_once_with(task.driver.power, task)
         mock_deploy_validate.assert_called_once_with(task.driver.deploy, task)
         mock_validate_traits.assert_called_once_with(task.node)
@@ -435,8 +547,7 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
         self.in_band_step = {
             'step': 'deploy_middle', 'priority': 30, 'interface': 'deploy'}
 
-    @mock.patch.object(deployments, 'LOG', autospec=True)
-    def test__do_next_deploy_step_none(self, mock_log):
+    def test__do_next_deploy_step_none(self):
         self._start_service()
         node = obj_utils.create_test_node(self.context, driver='fake-hardware')
         task = task_manager.TaskManager(self.context, node.uuid)
@@ -446,7 +557,6 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
 
         node.refresh()
         self.assertEqual(states.ACTIVE, node.provision_state)
-        self.assertEqual(2, mock_log.info.call_count)
 
     @mock.patch('ironic.drivers.modules.fake.FakeDeploy.execute_deploy_step',
                 autospec=True)
@@ -616,6 +726,47 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
         self.assertNotIn('agent_url', node.driver_internal_info)
         self.assertNotIn('agent_secret_token', node.driver_internal_info)
 
+    @mock.patch('ironic.drivers.modules.fake.FakeDeploy.execute_deploy_step',
+                autospec=True)
+    def test__do_next_deploy_step_dynamic(self, mock_execute):
+        # Simulate adding more steps in runtime.
+        driver_internal_info = {'deploy_step_index': None,
+                                'deploy_steps': self.deploy_steps[:1],
+                                'agent_url': 'url',
+                                'agent_secret_token': 'token'}
+        self._start_service()
+        node = obj_utils.create_test_node(
+            self.context, driver='fake-hardware',
+            driver_internal_info=driver_internal_info,
+            deploy_step={})
+
+        def _fake_execute(iface, task, step):
+            dii = task.node.driver_internal_info
+            dii['deploy_steps'] = self.deploy_steps
+            task.node.driver_internal_info = dii
+            task.node.save()
+
+        mock_execute.side_effect = _fake_execute
+
+        task = task_manager.TaskManager(self.context, node.uuid)
+        task.process_event('deploy')
+
+        deployments.do_next_deploy_step(task, 0)
+
+        # Deploying should be complete
+        node.refresh()
+        self.assertIsNone(node.last_error)
+        self.assertEqual(states.ACTIVE, node.provision_state)
+        self.assertEqual(states.NOSTATE, node.target_provision_state)
+        self.assertEqual({}, node.deploy_step)
+        self.assertNotIn('deploy_step_index', node.driver_internal_info)
+        self.assertIsNone(node.driver_internal_info['deploy_steps'])
+        mock_execute.assert_has_calls(
+            [mock.call(task.driver.deploy, task, self.deploy_steps[0]),
+             mock.call(task.driver.deploy, task, self.deploy_steps[1])])
+        self.assertNotIn('agent_url', node.driver_internal_info)
+        self.assertNotIn('agent_secret_token', node.driver_internal_info)
+
     @mock.patch('ironic.drivers.modules.fake.FakeDeploy.execute_deploy_step',
                 autospec=True)
     def test__do_next_deploy_step_fast_track(self, mock_execute):
@@ -654,20 +805,23 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
     @mock.patch.object(conductor_utils, 'LOG', autospec=True)
     @mock.patch('ironic.drivers.modules.fake.FakeDeploy.execute_deploy_step',
                 autospec=True)
-    def _do_next_deploy_step_execute_fail(self, exc, traceback,
+    def _do_next_deploy_step_execute_fail(self, exc, traceback, handled,
                                           mock_execute, mock_log):
         # When a deploy step fails, go to DEPLOYFAIL
         driver_internal_info = {'deploy_step_index': None,
                                 'deploy_steps': self.deploy_steps}
         self._start_service()
+
         node = obj_utils.create_test_node(
             self.context, driver='fake-hardware',
             driver_internal_info=driver_internal_info,
+            provision_state=states.DEPLOYFAIL if handled else states.DEPLOYING,
+            target_provision_state=states.ACTIVE,
+            last_error='existing error' if handled else None,
             deploy_step={})
         mock_execute.side_effect = exc
 
         task = task_manager.TaskManager(self.context, node.uuid)
-        task.process_event('deploy')
 
         deployments.do_next_deploy_step(task, 0)
 
@@ -675,20 +829,30 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
         node.refresh()
         self.assertEqual(states.DEPLOYFAIL, node.provision_state)
         self.assertEqual(states.ACTIVE, node.target_provision_state)
-        self.assertEqual({}, node.deploy_step)
-        self.assertNotIn('deploy_step_index', node.driver_internal_info)
-        self.assertIsNotNone(node.last_error)
+        if handled:
+            self.assertEqual('existing error', node.last_error)
+        else:
+            self.assertEqual({}, node.deploy_step)
+            self.assertNotIn('deploy_step_index', node.driver_internal_info)
+            self.assertIsNotNone(node.last_error)
+            self.assertIn(f"Deploy step deploy.{self.deploy_steps[0]['step']}",
+                          node.last_error)
+            mock_log.error.assert_called_once_with(mock.ANY,
+                                                   exc_info=traceback)
         self.assertFalse(node.maintenance)
         mock_execute.assert_called_once_with(mock.ANY, mock.ANY,
                                              self.deploy_steps[0])
-        mock_log.error.assert_called_once_with(mock.ANY, exc_info=traceback)
 
     def test_do_next_deploy_step_execute_ironic_exception(self):
         self._do_next_deploy_step_execute_fail(
-            exception.IronicException('foo'), False)
+            exception.IronicException('foo'), False, False)
 
     def test_do_next_deploy_step_execute_exception(self):
-        self._do_next_deploy_step_execute_fail(Exception('foo'), True)
+        self._do_next_deploy_step_execute_fail(Exception('foo'), True, False)
+
+    def test_do_next_deploy_step_execute_handled_exception(self):
+        self._do_next_deploy_step_execute_fail(
+            exception.IronicException('foo'), False, True)
 
     @mock.patch('ironic.drivers.modules.fake.FakeDeploy.execute_deploy_step',
                 autospec=True)
@@ -852,6 +1016,34 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
         mock_execute.assert_called_once_with(
             mock.ANY, mock.ANY, self.deploy_steps[0])
 
+    def _test_do_next_deploy_step_handles(self, start_state):
+        node = obj_utils.create_test_node(
+            self.context, driver='fake-hardware',
+            provision_state=start_state,
+            driver_internal_info={
+                'deploy_steps': [
+                    {
+                        'step': 'hold',
+                        'priority': 10,
+                        'interface': 'deploy'
+                    }
+                ],
+                'deploy_step_index': None},
+            clean_step=None)
+        with task_manager.acquire(
+                self.context, node.uuid, shared=False) as task:
+            deployments.do_next_deploy_step(task, 0)
+        node.refresh()
+        self.assertEqual(states.DEPLOYHOLD, node.provision_state)
+
+    def test_do_next_deploy_step_handles_hold_from_active(self):
+        # Prior step/action was out of the conductor
+        self._test_do_next_deploy_step_handles(states.DEPLOYING)
+
+    def test_do_next_deploy_step_handles_hold_from_wait(self):
+        # Prior step was async in a wait state
+        self._test_do_next_deploy_step_handles(states.DEPLOYWAIT)
+
     @mock.patch.object(deployments, 'do_next_deploy_step', autospec=True)
     @mock.patch.object(conductor_steps, '_get_steps', autospec=True)
     def _continue_node_deploy(self, mock__get_steps, mock_next_step,
@@ -964,7 +1156,7 @@ class StoreConfigDriveTestCase(db_base.DbTestCase):
                           group='conductor')
         mock_swift.return_value.get_temp_url.return_value = 'http://1.2.3.4'
 
-        deployments._store_configdrive(self.node, b'foo')
+        deployments._store_configdrive(self.node, 'foo')
 
         mock_swift.assert_called_once_with()
         mock_swift.return_value.create_object.assert_called_once_with(
@@ -975,6 +1167,37 @@ class StoreConfigDriveTestCase(db_base.DbTestCase):
         self.node.refresh()
         self.assertEqual(expected_instance_info, self.node.instance_info)
 
+    @mock.patch.object(conductor_utils, 'build_configdrive', autospec=True)
+    def test_store_configdrive_swift_build(self, mock_cd, mock_swift):
+        container_name = 'foo_container'
+        timeout = 123
+        expected_obj_name = 'configdrive-%s' % self.node.uuid
+        expected_obj_header = {'X-Delete-After': str(timeout)}
+        expected_instance_info = {'configdrive': 'http://1.2.3.4'}
+
+        mock_cd.return_value = 'fake'
+
+        # set configs and mocks
+        CONF.set_override('configdrive_use_object_store', True,
+                          group='deploy')
+        CONF.set_override('configdrive_swift_container', container_name,
+                          group='conductor')
+        CONF.set_override('deploy_callback_timeout', timeout,
+                          group='conductor')
+        mock_swift.return_value.get_temp_url.return_value = 'http://1.2.3.4'
+
+        deployments._store_configdrive(self.node, {'meta_data': {}})
+
+        mock_swift.assert_called_once_with()
+        mock_swift.return_value.create_object.assert_called_once_with(
+            container_name, expected_obj_name, mock.ANY,
+            object_headers=expected_obj_header)
+        mock_swift.return_value.get_temp_url.assert_called_once_with(
+            container_name, expected_obj_name, timeout)
+        self.node.refresh()
+        self.assertEqual(expected_instance_info, self.node.instance_info)
+        mock_cd.assert_called_once_with(self.node, {'meta_data': {}})
+
     def test_store_configdrive_swift_no_deploy_timeout(self, mock_swift):
         container_name = 'foo_container'
         expected_obj_name = 'configdrive-%s' % self.node.uuid
@@ -992,7 +1215,7 @@ class StoreConfigDriveTestCase(db_base.DbTestCase):
                           group='conductor')
         mock_swift.return_value.get_temp_url.return_value = 'http://1.2.3.4'
 
-        deployments._store_configdrive(self.node, b'foo')
+        deployments._store_configdrive(self.node, 'foo')
 
         mock_swift.assert_called_once_with()
         mock_swift.return_value.create_object.assert_called_once_with(
@@ -1019,7 +1242,7 @@ class StoreConfigDriveTestCase(db_base.DbTestCase):
                           group='conductor')
         mock_swift.return_value.get_temp_url.return_value = 'http://1.2.3.4'
 
-        deployments._store_configdrive(self.node, b'foo')
+        deployments._store_configdrive(self.node, 'foo')
 
         mock_swift.assert_called_once_with()
         mock_swift.return_value.create_object.assert_called_once_with(
